@@ -1,7 +1,6 @@
 #ifndef YACAS_PATTERNCLASS_H
 #define YACAS_PATTERNCLASS_H
 
-#include "lisptype.h"
 #include "lispobject.h"
 #include "genericobject.h"
 #include "patterns.h"
@@ -10,7 +9,7 @@
 /// Wrapper for YacasPatternPredicateBase.
 /// This class allows a YacasPatternPredicateBase to be put in a
 /// LispGenericObject.
-class PatternClass : public GenericClass, NonCopyable
+class PatternClass final: public GenericClass, NonCopyable
 {
 public:
   PatternClass(YacasPatternPredicateBase* aPatternMatcher);
@@ -21,7 +20,7 @@ public:
   bool Matches(LispEnvironment& aEnvironment,
                       LispPtr* aArguments);
 
-  const LispChar * TypeName() const override;
+  const char* TypeName() const override;
 
 protected:
   YacasPatternPredicateBase* iPatternMatcher;
