@@ -1,33 +1,31 @@
 
 #include "yacas/lispenvironment.h"
-#include "yacas/lispeval.h"
+#include "yacas/errors.h"
 #include "yacas/lispatom.h"
-#include "yacas/standard.h"
+#include "yacas/lispeval.h"
 #include "yacas/lispuserfunc.h"
 #include "yacas/mathuserfunc.h"
-#include "yacas/errors.h"
+#include "yacas/standard.h"
 
 // we need this only for digits_to_bits
 #include "yacas/numbers.h"
 
-LispEnvironment::LispEnvironment(
-                    YacasCoreCommands& aCoreCommands,
-                    LispUserFunctions& aUserFunctions,
-                    LispGlobal& aGlobals,
-                    LispHashTable& aHashTable,
-                    std::ostream& aOutput,
-                    LispPrinter& aPrinter,
-                    LispOperators &aPreFixOperators,
-                    LispOperators &aInFixOperators,
-                    LispOperators &aPostFixOperators,
-                    LispOperators &aBodiedOperators,
-                    LispIdentifiers& protected_symbols,
-                    LispInput*    aCurrentInput)
-    :
-    iPrecision(10),  // default user precision of 10 decimal digits
-    iBinaryPrecision(34),  // same as 34 bits
+LispEnvironment::LispEnvironment(YacasCoreCommands& aCoreCommands,
+                                 LispUserFunctions& aUserFunctions,
+                                 LispGlobal& aGlobals,
+                                 LispHashTable& aHashTable,
+                                 std::ostream& aOutput,
+                                 LispPrinter& aPrinter,
+                                 LispOperators& aPreFixOperators,
+                                 LispOperators& aInFixOperators,
+                                 LispOperators& aPostFixOperators,
+                                 LispOperators& aBodiedOperators,
+                                 LispIdentifiers& protected_symbols,
+                                 LispInput* aCurrentInput) :
+    iPrecision(10),       // default user precision of 10 decimal digits
+    iBinaryPrecision(34), // same as 34 bits
     iInputDirectories(),
-    //iCleanup(),
+    // iCleanup(),
     iEvalDepth(0),
     iMaxEvalDepth(1000),
     stop_evaluation(false),
@@ -70,24 +68,24 @@ LispEnvironment::LispEnvironment(
     iXmlTokenizer(),
     iCurrentTokenizer(&iDefaultTokenizer)
 {
-    iTrue = LispAtom::New(*this,"True");
-    iFalse = LispAtom::New(*this,"False");
+    iTrue = LispAtom::New(*this, "True");
+    iFalse = LispAtom::New(*this, "False");
 
     Protect(iTrue->String());
     Protect(iFalse->String());
 
-    iEndOfFile    = LispAtom::New(*this,"EndOfFile");
-    iEndStatement = LispAtom::New(*this,";");
-    iProgOpen     = LispAtom::New(*this,"[");
-    iProgClose    = LispAtom::New(*this,"]");
-    iNth          = LispAtom::New(*this,"Nth");
-    iBracketOpen  = LispAtom::New(*this,"(");
-    iBracketClose = LispAtom::New(*this,")");
-    iListOpen     = LispAtom::New(*this,"{");
-    iListClose    = LispAtom::New(*this,"}");
-    iComma        = LispAtom::New(*this,",");
-    iList         = LispAtom::New(*this,"List");
-    iProg         = LispAtom::New(*this,"Prog");
+    iEndOfFile = LispAtom::New(*this, "EndOfFile");
+    iEndStatement = LispAtom::New(*this, ";");
+    iProgOpen = LispAtom::New(*this, "[");
+    iProgClose = LispAtom::New(*this, "]");
+    iNth = LispAtom::New(*this, "Nth");
+    iBracketOpen = LispAtom::New(*this, "(");
+    iBracketClose = LispAtom::New(*this, ")");
+    iListOpen = LispAtom::New(*this, "{");
+    iListClose = LispAtom::New(*this, "}");
+    iComma = LispAtom::New(*this, ",");
+    iList = LispAtom::New(*this, "List");
+    iProg = LispAtom::New(*this, "Prog");
 
     Protect(iList->String());
     Protect(iProg->String());
@@ -98,7 +96,6 @@ LispEnvironment::LispEnvironment(
     PushLocalFrame(true);
 }
 
-
 LispEnvironment::~LispEnvironment()
 {
     delete iEvaluator;
@@ -107,8 +104,8 @@ LispEnvironment::~LispEnvironment()
 
 void LispEnvironment::SetPrecision(int aPrecision)
 {
-    iPrecision = aPrecision;  // precision in decimal digits
-  iBinaryPrecision = digits_to_bits(aPrecision, BASE10);  // in bits
+    iPrecision = aPrecision; // precision in decimal digits
+    iBinaryPrecision = digits_to_bits(aPrecision, BASE10); // in bits
 }
 
 int LispEnvironment::GetUniqueId()
@@ -116,14 +113,16 @@ int LispEnvironment::GetUniqueId()
     return iLastUniqueId++;
 }
 
-
 LispPtr* LispEnvironment::FindLocal(const LispString* aVariable)
 {
     assert(!_local_frames.empty());
 
     std::size_t last = _local_vars.size();
 
-    for (std::vector<LocalVariableFrame>::const_reverse_iterator f = _local_frames.rbegin(); f != _local_frames.rend(); ++f) {
+    for (std::vector<LocalVariableFrame>::const_reverse_iterator f =
+             _local_frames.rbegin();
+         f != _local_frames.rend();
+         ++f) {
         const std::size_t first = f->first;
         for (std::size_t i = last; i > first; --i)
             if (_local_vars[i - 1].var == aVariable)
@@ -137,9 +136,11 @@ LispPtr* LispEnvironment::FindLocal(const LispString* aVariable)
     return nullptr;
 }
 
-void LispEnvironment::SetVariable(const LispString* aVariable, LispPtr& aValue, bool aGlobalLazyVariable)
+void LispEnvironment::SetVariable(const LispString* aVariable,
+                                  LispPtr& aValue,
+                                  bool aGlobalLazyVariable)
 {
-    if (LispPtr *local = FindLocal(aVariable)) {
+    if (LispPtr* local = FindLocal(aVariable)) {
         *local = aValue;
         return;
     }
@@ -152,7 +153,9 @@ void LispEnvironment::SetVariable(const LispString* aVariable, LispPtr& aValue,
     if (i != iGlobals.end())
         i->second = LispGlobalVariable(aValue);
     else
-        i = iGlobals.insert(std::make_pair(aVariable, LispGlobalVariable(aValue))).first;
+        i = iGlobals
+                .insert(std::make_pair(aVariable, LispGlobalVariable(aValue)))
+                .first;
 
     if (aGlobalLazyVariable)
         i->second.SetEvalBeforeReturn(true);
@@ -173,7 +176,9 @@ void LispEnvironment::GetVariable(const LispString* aVariable, LispPtr& aResult)
         LispGlobalVariable* l = &i->second;
         if (l->iEvalBeforeReturn) {
             iEvaluator->Eval(*this, aResult, l->iValue);
-            // re-lookup the global variable, as this pointer might now be invalid due to the evaluation actually changing the global itself.
+            // re-lookup the global variable, as this pointer might now be
+            // invalid due to the evaluation actually changing the global
+            // itself.
             l = &iGlobals.find(aVariable)->second;
 
             l->iValue = aResult;
@@ -205,7 +210,8 @@ void LispEnvironment::PopLocalFrame()
 {
     assert(!_local_frames.empty());
 
-    _local_vars.erase(_local_vars.begin() + _local_frames.back().first, _local_vars.end());
+    _local_vars.erase(_local_vars.begin() + _local_frames.back().first,
+                      _local_vars.end());
     _local_frames.pop_back();
 }
 
@@ -224,20 +230,39 @@ void LispEnvironment::CurrentLocals(LispPtr& aResult)
 
     std::size_t last = _local_vars.size();
 
-    for (std::vector<LocalVariableFrame>::const_reverse_iterator f = _local_frames.rbegin(); f != _local_frames.rend(); ++f) {
+    for (std::vector<LocalVariableFrame>::const_reverse_iterator f =
+             _local_frames.rbegin();
+         f != _local_frames.rend();
+         ++f) {
         const std::size_t first = f->first;
         for (std::size_t i = last; i > first; --i)
-            locals = LispObjectAdder(LispAtom::New(*this, *_local_vars[i - 1].var)) + LispObjectAdder(locals);
+            locals =
+                LispObjectAdder(LispAtom::New(*this, *_local_vars[i - 1].var)) +
+                LispObjectAdder(locals);
 
         if (f->fenced)
             break;
 
         last = first;
     }
-    aResult = LispSubList::New(LispObjectAdder(iList->Copy()) + LispObjectAdder(locals));
+    aResult = LispSubList::New(LispObjectAdder(iList->Copy()) +
+                               LispObjectAdder(locals));
 }
 
+void LispEnvironment::GlobalVariables(LispPtr& aResult)
+{
+    LispPtr vars(iList->Copy());
+    LispIterator tail(vars);
+    ++tail;
 
+    for (const auto& p: iGlobals) {
+        if (p.first->front() == '$' || p.first->front() == '%')
+            continue;
+        *tail = LispAtom::New(*this, *p.first);
+        ++tail;
+    }
+    aResult = LispSubList::New(vars);
+}
 
 LispPrinter& LispEnvironment::CurrentPrinter()
 {
@@ -266,7 +291,6 @@ LispOperators& LispEnvironment::Bodied()
     return iBodiedOperators;
 }
 
-
 LispInput* LispEnvironment::CurrentInput()
 {
     return iCurrentInput;
@@ -287,21 +311,19 @@ void LispEnvironment::SetCurrentOutput(std::ostream& aOutput)
     iCurrentOutput = &aOutput;
 }
 
-
-
 LispUserFunction* LispEnvironment::UserFunction(LispPtr& aArguments)
 {
     auto i = iUserFunctions.find(aArguments->String());
     if (i != iUserFunctions.end()) {
         LispMultiUserFunction* multiUserFunc = &i->second;
-        int arity = InternalListLength(aArguments)-1;
-        return  multiUserFunc->UserFunc(arity);
+        int arity = InternalListLength(aArguments) - 1;
+        return multiUserFunc->UserFunc(arity);
     }
     return nullptr;
 }
 
-
-LispUserFunction* LispEnvironment::UserFunction(const LispString* aName, int aArity)
+LispUserFunction* LispEnvironment::UserFunction(const LispString* aName,
+                                                int aArity)
 {
     auto i = iUserFunctions.find(aName);
     if (i != iUserFunctions.end())
@@ -310,8 +332,6 @@ LispUserFunction* LispEnvironment::UserFunction(const LispString* aName, int aAr
     return nullptr;
 }
 
-
-
 void LispEnvironment::UnFenceRule(const LispString* aOperator, int aArity)
 {
     if (Protected(aOperator))
@@ -360,33 +380,31 @@ void LispEnvironment::DeclareRuleBase(const LispString* aOperator,
         */
 
     // add an operator with this arity to the multiuserfunc.
-    BranchingUserFunction* newFunc = 
-            aListed
-            ? new ListedBranchingUserFunction(aParameters) 
-            : new BranchingUserFunction(aParameters);
+    BranchingUserFunction* newFunc =
+        aListed ? new ListedBranchingUserFunction(aParameters)
+                : new BranchingUserFunction(aParameters);
 
     multiUserFunc->DefineRuleBase(newFunc);
 }
 
-void LispEnvironment::DeclareMacroRuleBase(const LispString* aOperator, LispPtr& aParameters, int aListed)
+void LispEnvironment::DeclareMacroRuleBase(const LispString* aOperator,
+                                           LispPtr& aParameters,
+                                           int aListed)
 {
     if (Protected(aOperator))
         throw LispErrProtectedSymbol(*aOperator);
 
     LispMultiUserFunction* multiUserFunc = MultiUserFunction(aOperator);
 
-    MacroUserFunction* newFunc = 
-            aListed 
-            ? new ListedMacroUserFunction(aParameters) 
-            : new MacroUserFunction(aParameters);
+    MacroUserFunction* newFunc = aListed
+                                     ? new ListedMacroUserFunction(aParameters)
+                                     : new MacroUserFunction(aParameters);
 
     multiUserFunc->DefineRuleBase(newFunc);
 }
 
-
-
-
-LispMultiUserFunction* LispEnvironment::MultiUserFunction(const LispString* aOperator)
+LispMultiUserFunction*
+LispEnvironment::MultiUserFunction(const LispString* aOperator)
 {
     auto i = iUserFunctions.find(aOperator);
 
@@ -394,12 +412,12 @@ LispMultiUserFunction* LispEnvironment::MultiUserFunction(const LispString* aOpe
         return &i->second;
 
     LispMultiUserFunction newMulti;
-    return &iUserFunctions.insert(std::make_pair(aOperator, newMulti)).first->second;
-    //SetAssociation(newMulti, aOperator);
+    return &iUserFunctions.insert(std::make_pair(aOperator, newMulti))
+                .first->second;
 }
 
-
-void LispEnvironment::HoldArgument(const LispString*  aOperator, const LispString* aVariable)
+void LispEnvironment::HoldArgument(const LispString* aOperator,
+                                   const LispString* aVariable)
 {
     auto i = iUserFunctions.find(aOperator);
 
@@ -426,8 +444,10 @@ bool LispEnvironment::Protected(const LispString* symbol) const
     return protected_symbols.find(symbol) != protected_symbols.end();
 }
 
-void LispEnvironment::DefineRule(const LispString* aOperator,int aArity,
-                                 int aPrecedence, LispPtr& aPredicate,
+void LispEnvironment::DefineRule(const LispString* aOperator,
+                                 int aArity,
+                                 int aPrecedence,
+                                 LispPtr& aPredicate,
                                  LispPtr& aBody)
 {
     if (Protected(aOperator))
@@ -449,22 +469,21 @@ void LispEnvironment::DefineRule(const LispString* aOperator,int aArity,
 
     // Declare a new evaluation rule
 
-
-    if (IsTrue(*this, aPredicate))
-    {
-//        printf("FastPredicate on %s\n",aOperator->String());
+    if (IsTrue(*this, aPredicate)) {
+        //        printf("FastPredicate on %s\n",aOperator->String());
         userFunc->DeclareRule(aPrecedence, aBody);
-    }
-    else
-        userFunc->DeclareRule(aPrecedence, aPredicate,aBody);
+    } else
+        userFunc->DeclareRule(aPrecedence, aPredicate, aBody);
 }
 
-void LispEnvironment::DefineRulePattern(const LispString* aOperator,int aArity,
-                                        int aPrecedence, LispPtr& aPredicate,
+void LispEnvironment::DefineRulePattern(const LispString* aOperator,
+                                        int aArity,
+                                        int aPrecedence,
+                                        LispPtr& aPredicate,
                                         LispPtr& aBody)
 {
-//    if (Protected(aOperator))
-//        throw LispErrProtectedSymbol(*aOperator);
+    //    if (Protected(aOperator))
+    //        throw LispErrProtectedSymbol(*aOperator);
 
     // Find existing multiuser func.
     auto i = iUserFunctions.find(aOperator);
@@ -481,29 +500,34 @@ void LispEnvironment::DefineRulePattern(const LispString* aOperator,int aArity,
         throw LispErrCreatingRule();
 
     // Declare a new evaluation rule
-    userFunc->DeclarePattern(aPrecedence, aPredicate,aBody);
+    userFunc->DeclarePattern(aPrecedence, aPredicate, aBody);
 }
 
-void LispEnvironment::SetCommand(YacasEvalCaller aEvaluatorFunc, const char* aString,int aNrArgs,int aFlags)
+void LispEnvironment::SetCommand(YacasEvalCaller aEvaluatorFunc,
+                                 const char* aString,
+                                 int aNrArgs,
+                                 int aFlags)
 {
-  const LispString* name = HashTable().LookUp(aString);
-  YacasEvaluator eval(aEvaluatorFunc,aNrArgs,aFlags);
-  auto i = iCoreCommands.find(name);
-  if (i != iCoreCommands.end())
-      i->second = eval;
-  else
-      iCoreCommands.insert(std::make_pair(name, eval));
+    const LispString* name = HashTable().LookUp(aString);
+    YacasEvaluator eval(aEvaluatorFunc, aNrArgs, aFlags);
+    auto i = iCoreCommands.find(name);
+    if (i != iCoreCommands.end())
+        i->second = eval;
+    else
+        iCoreCommands.insert(std::make_pair(name, eval));
 }
 
 void LispEnvironment::RemoveCoreCommand(char* aString)
 {
-  iCoreCommands.erase(HashTable().LookUp(aString));
+    iCoreCommands.erase(HashTable().LookUp(aString));
 }
 
-LispLocalEvaluator::LispLocalEvaluator(LispEnvironment& aEnvironment,LispEvaluatorBase* aNewEvaluator)
-  : iPreviousEvaluator(aEnvironment.iEvaluator),iEnvironment(aEnvironment)
+LispLocalEvaluator::LispLocalEvaluator(LispEnvironment& aEnvironment,
+                                       LispEvaluatorBase* aNewEvaluator) :
+    iPreviousEvaluator(aEnvironment.iEvaluator),
+    iEnvironment(aEnvironment)
 {
-  aEnvironment.iEvaluator = aNewEvaluator;
+    aEnvironment.iEvaluator = aNewEvaluator;
 }
 LispLocalEvaluator::~LispLocalEvaluator()
 {
@@ -511,15 +535,14 @@ LispLocalEvaluator::~LispLocalEvaluator()
     iEnvironment.iEvaluator = iPreviousEvaluator;
 }
 
-LispLocalTrace::LispLocalTrace(LispUserFunction* aUserFunc) : iUserFunc(aUserFunc)
+LispLocalTrace::LispLocalTrace(LispUserFunction* aUserFunc) :
+    iUserFunc(aUserFunc)
 {
-  if (iUserFunc!=nullptr)
-    iUserFunc->Trace();
+    if (iUserFunc != nullptr)
+        iUserFunc->Trace();
 }
 LispLocalTrace::~LispLocalTrace()
 {
-  if (iUserFunc!=nullptr)
-    iUserFunc->UnTrace();
+    if (iUserFunc != nullptr)
+        iUserFunc->UnTrace();
 }
-
-
